/* Fixed point sin(), cos(), and product (in fixed.h). */

#include "fixed.h"
#include "sintab.h"

/* Fixed point sin().  Domain is the real line, represented as 32 bit
fixed point integers with the binary point in the middle. */

Int32 fsin(Int32 i)
{
	Int32 s, r;

	/* Get into [0 .. pi / 2].  First make it non-negative. */

	s = 0;
	if (i < 0) {
		i = -i;
		s = 1;
	}

	/* mod 2 * pi.  I don't know how many times you have to go
	through the loop to equal the time of calling product, but
	it's probably many, and lots of times i will be close to
	zero anyway. */

	if (i <= MANYPI) {
		while (i >= TWOPI) {
			i -= TWOPI;
		}
	} else {
		i = product(i, ITWOPI);
		i &= 0xFFFF;
		i = product(i, TWOPI);
	}

	/* Check for > pi. */

	if (i > PI) {
		i -= PI;
		s ^= 1;
	}

	/* Check for > pi / 2. */

	if (i > HALFPI) {
		i = PI - i;
	}

	/* Map [0 .. pi / 2] into [0 .. N]. */

	i = product(i, TABFACT);
	i >>= 16;

	/* Get the result for this quadrant.  (We could halve the size
	of the table by doing
		if (i == N) {
			r = 0x10000;
		} else {
			r = Sintab[i];
		}
	and getting rid of the upper halfword of each entry,
	since the result is always < 1 except for that one value.) */

if (i < 0 || i > 252) return 0;
	r = Sintab[i];

	/* Map it appropriately for the original quadrant. */

	if (s) {
		r = -r;
	}
	return r;
}

/* Fixed point cos(). */

Int32 fcos(Int32 i)
{
	return fsin(i + HALFPI);
}
