#include <PalmOS.h>
#include "fixed.h"
#include "rsrc.h"

#if 0
#include <stdarg.h>

void msg(const char *str, ...)
{
	char buf[100];
	va_list ap;
	static int row = 15;

	va_start(ap, str);
	StrVPrintF(buf, str, ap);
	WinDrawChars(buf, StrLen(buf), 0, row);
	va_end(ap);
	row += 10;
}
#endif

#define M_PI 3.14159265358979323846

/* Circumferences, i.e. the number of teeth in the gears. */

int C1, C2;

/* The fraction of the radius of the inner gear where the pen is (percent). */

int K;

/* Number of ticks per 2 pi.  Don't change this, because the fsin()
routine is only about this accurate. */

#define NTICKS 200

/* Center of the display. */

#define XORIGIN 80
#define YORIGIN 87
#define RADIUS 72

/* UI state variables. */

extern int Changed;
extern int Done;

extern FieldType *Active_field;
extern int *Active_var;

int gcd(int a, int b)
{
	int q;

	while (b != 0) {
		q = a % b;
		a = b;
		b = q;
	}
	return a;
}

#define FSIZE 5

void set_field(FieldType *field, char *str)
{
	int i;
	MemHandle h;
	MemPtr buf;

	i = StrLen(str);
	if (i >= FSIZE) {
		str[FSIZE - 1] = '\0';
	}

	/* Get the handle for the string and unlock it by removing it from
	the field. */

	h = FldGetTextHandle(field);
	FldSetTextHandle(field, NULL);

	/* Allocate or resize. */

	if (h == NULL) {
		h = MemHandleNew(FSIZE);
	} else {
		MemHandleResize(h, FSIZE);
	}

	/* Lock it (which gets the pointer), set it, and unlock it. */

	buf = MemHandleLock(h);
	MemMove(buf, str, StrLen(str) + 1);
	MemHandleUnlock(h);

	/* Update the text in the field. */

	FldSetTextHandle(field, h);
	FldDrawField(field);
}

void update_fields()
{
	char buf[30];

	StrPrintF(buf, "%d", *Active_var);
	set_field(Active_field, buf);
}

void osc()
{
	Int32 j, t, x, y;
	double f, r, R;
	RectangleType rect, cliprect;
	static int i = 0;
	static int n = 0;
	static Int32 dt, c, rdiff, rfact;
	static Coord x1, y1, x2, y2;

	/* Get the old clip rect and set the new one. */

	WinGetClip(&cliprect);
	rect.topLeft.x = 0;
	rect.topLeft.y = 15;
	rect.extent.x = 160;
	rect.extent.y = 160 - 15;
	WinSetClip(&rect);

	/* If the settings changed, clear the window and recalculate
	all the loop constants. */

	if (Changed) {
		Changed = false;

		/* Clear the window. */

		WinEraseRectangle(&rect, 0);

		/* Calculate radii. */

		R = C1 / (2. * M_PI);
		r = C2 / (2. * M_PI);

		/* Normalize. */

		f = R;
		R /= f;
		r /= f;

		/* The total number of ticks is NTICKS times the number of
		times the inner wheel goes around in the big wheel. */

		n = C2 / gcd(C1, C2);
		n *= NTICKS;

		/* Set fixed point variables used in the loop. */

		dt = FIX(2. * M_PI / NTICKS);
		c = FIX((K / 100.) * r * RADIUS);
		rdiff = FIX((R - r) * RADIUS);
		rfact = FIX(R / r - 1.);

		/* Set the initial point. */

		i = 0;
		x = rdiff + c;
		y = 0;
		x1 = FIX2INT(x) + XORIGIN;
		y1 = FIX2INT(y) + YORIGIN;
	}

	/* Draw lines until either we finish or something happens. */

	while (i < n) {
		j = i++;
		j <<= 16;
		t = product(j, dt);

		/* Hypotrochoid (moving wheel inside fixed wheel). */

		x = product(rdiff, fcos(t));
		x += product(c, fcos(product(rfact, t)));
		y = product(rdiff, fsin(t));
		y -= product(c, fsin(product(rfact, t)));

		x2 = FIX2INT(x) + XORIGIN;
		y2 = FIX2INT(y) + YORIGIN;
		WinDrawLine(x1, y1, x2, y2);
		x1 = x2;
		y1 = y2;

		if ((i & 7) == 0 && EvtEventAvail()) {
			break;
		}
	}
	if (i >= n) {
		Done = true;
	}

	/* Restore the previous clip rect. */

	WinSetClip(&cliprect);
}
