#include <PalmOS.h>
#include "rsrc.h"

/* Oscillator variables and functionality. */

extern int C1, C2, K;
extern void update_fields();
extern void osc();

FieldType *Active_field;
int *Active_var;
int Inc;

ControlType *Active_ctl;
Char Buttonlabel[3][10];

/* Granularity of the increments & decrements for C1, C2, & K. */

#define CINC 1
#define KINC 1

/* This is set when there's nothing to do and we can go to sleep. */

int Done = false;

/* This gets set when something in the UI changes. */

int Changed;

/* Update the variable from the field's text. */

void update_var()
{
	Int32 i;
	Char *s;

	s = FldGetTextPtr(Active_field);
	i = StrAToI(s);
	if (i > 0) {
		*Active_var = i;
	}
}

Boolean MainFormHandleEvent(EventPtr event)
{
	UInt16 cid;
	Char *s;
	Boolean handled = false;
	ControlType *ctl;

	switch (event->eType) {

	case ctlSelectEvent:
		cid = event->data.ctlSelect.controlID;
		ctl = event->data.ctlSelect.pControl;
		switch (cid) {

		case OButton:
			FrmPopupForm(OForm);
			Active_var = &C1;
			Inc = CINC;
			Active_ctl = ctl;
			handled = true;
			break;

		case oButton:
			FrmPopupForm(oForm);
			Active_var = &C2;
			Inc = CINC;
			Active_ctl = ctl;
			handled = true;
			break;

		case rButton:
			FrmPopupForm(rForm);
			Active_var = &K;
			Inc = KINC;
			Active_ctl = ctl;
			handled = true;
			break;

		case ODone:
		case oDone:
		case rDone:
			update_var();
			s = &Buttonlabel[(cid / 1000) - 2][0];
			StrPrintF(s, "%d", *Active_var);
			FrmReturnToForm(MainForm);
			CtlSetLabel(Active_ctl, s);
			Active_field = NULL;
			handled = true;
			break;
		}
		break;

	case ctlRepeatEvent:
		Changed = true;
		update_var();
		switch (event->data.ctlRepeat.controlID) {

		case SpiroCirc1Inc:
			C1 += Inc;
			if (C1 > 100) C1 = 100;
			break;

		case SpiroCirc1Dec:
			C1 -= Inc;
			if (C1 < 1) C1 = CINC;
			break;

		case SpiroCirc2Inc:
			C2 += Inc;
			if (C2 > 100) C2 = 100;
			break;

		case SpiroCirc2Dec:
			C2 -= Inc;
			if (C2 < 1) C2 = CINC;
			break;

		case SpiroCirc3Inc:
			K += Inc;
			if (K > 100) K = 100;
			break;

		case SpiroCirc3Dec:
			K -= Inc;
			if (K < 1) K = KINC;
			break;

		default:
			Changed = false;
			break;
		}
		if (Changed) {
			update_fields();
			Done = false;
		}
		break;

	case frmOpenEvent:
		FrmDrawForm(FrmGetActiveForm());
		if (Active_field) {
			update_fields();
		}
		Changed = true;
		Done = false;
		handled = true;
		break;

	case nilEvent:
		if (!Active_field) {
			osc();
		}
		handled = true;
		break;
	}

	return handled;
}

Boolean ApplicationHandleEvent(EventPtr event)
{
	FormType *form;
	UInt16 formId, fieldId;

	if (event->eType != frmLoadEvent) {
		return false;
	}

	formId = event->data.frmLoad.formID;
	form = FrmInitForm(formId);
	FrmSetActiveForm(form);

	if (formId != MainForm &&
	    formId != OForm && formId != oForm && formId != rForm) {
		return true;
	}
	FrmSetEventHandler(form, MainFormHandleEvent);

	if (formId == MainForm) {
		return true;
	}

	/* The field IDs in rsrc.h must be one greater than the
	corresponding form IDs. */

	fieldId = formId + 1;
	Active_field = FrmGetObjectPtr(form, FrmGetObjectIndex(form, fieldId));

	return true;
}

UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
	Int32 t;
	EventType event;
	Err error;

	if (cmd != sysAppLaunchCmdNormalLaunch) {
		return 0;
	}

	C1 = 50;
	C2 = 30;
	K = 50;
	Active_field = NULL;

	FrmGotoForm(MainForm);

	do {
		t = 20;
		if (Done) {
			t = -1;
		}
		EvtGetEvent(&event, t);

		if (SysHandleEvent(&event)) {
			continue;
		}
		if (MenuHandleEvent(NULL, &event, &error)) {
			continue;
		}
		if (ApplicationHandleEvent(&event)) {
			continue;
		}
		FrmDispatchEvent(&event);
	} while (event.eType != appStopEvent);

	return 0;
}
